// New Widget system.
// Well... errr... not much easier, but it is certainly a lot more generic.
//
// An explanation on how stuff works with this:
// Widgets get organised in a tree. Everything (including windows and the root
// window) are considered widgets with this.
//
// All communication with the user interface goes using events, so it's
// wisest to make an event handler for each widget which actually wants events.
// Widgets which don't get an event handler won't be susceptible to events.
//
// Keybindings need to be moved around useing events, so make sure that
// they are handled correctly when a widget gets created/destroyed/focussed.
//
// A few words of warning:
// - The first surface in a list is considered to be on top.
// - The root window can never be on top (the one with parent NULL).
#ifndef NEW_WIDGET
#define NEW_WIDGET

//////////////////////////////////////////////////////////////////////
//
// Event related stuff.
//

enum {
	E_CREATE_WIDGET = 1,
	E_DESTROY_WIDGET = 2,
	E_UPDATE_WIDGET = 3,
	E_MOUSE = 4,
	E_KEY = 5,
	E_MESSAGE = 6,
};

typedef struct Event {
	byte		type;
	uint32	info;
	void*		unknown;
} Event;

typedef struct Widget Widget;

typedef uint32	(*EventHandler)(Event* e, Widget* w);

//////////////////////////////////////////////////////////////////////
//
// Everything generic related to widgets.
// i.e. scrollbar code has /nothing/ to do here.
//

// Widget flags.
enum {
	WANTS_MOUSE = 1,
	REDRAW_MARK = 2,									// Tells the art system which widgets to redraw -> performance.
	XY_ABSOLUTE = 4,									// Set this for a window
	IS_LIST 		= 8,
};

///
/// This struct is used for each and every widget.
/// That includes the root-window, windows, buttons, scrollbars (in effect 4 widgets (counting buttons)) and viewports,
/// each and every item in a depot visible, etc.
///
struct Widget {
	Rect								xy;						//! The coordinates of the widget.
	byte								flags;				//! Static information about this widget. This should not change during its lifetime.
	void*								state;				// Some widgets require a lot of info, others have nearly none.
	
	void*								graphic_cache;// This contains the /current/ graphic of this widget and all visible child widgets.
																		//  Should be redone if the graphical state of this widget or one of its children changes.
	Widget*							parent;
	Widget*							child_list;		// Pointer to the first child of this widget.
	Widget*							next;					// Next widget in the chain.

	EventHandler*				on_event;			//! Pointer to a function which handles all events for this widget.
};

VARDEF	Widget*				_widget_tree;

// new_widgets.c
//! Add a child widget, to parent w.
inline Widget*	CreateWidget (Widget* w, byte flags, EventHandler* ev);
inline void			DeleteWidget (Widget* w);

inline void			PutWidgetOnTop (Widget* w);
inline void			MarkForRedraw (Widget* w);						//! Marks a widget for redrawing. Sets its parents too, call this from
																											//!  an event handler.
inline void			HandleRedraw (Widget* w);
inline Widget*	FindWidgetXY (Widget* w, Point xy);		//! Finds a widget at the specified coordinate, comes in useful for events.

// Check whether a point falls in a given area (comes in handy for mouse events).
static inline bool InArea (Rect area, Point xy)
{
	return (((xy.x >= area.left) && (xy.x <= area.right)) &&
					((xy.y >= area.top) && (xy.y <= area.bottom))) ? true : false;
};

// Checks if a rect A is (partly) located in rect B.
static inline bool RectInRect (Rect a, rect b)
{
	return (((a.left <= b.right) || (a.right >= b.left)) &&
					((a.top <= b.bottom) || (a.bottom >= b.top))) ? true : false;
};

// Figure out which part of rect b is visible in rect a and return that.
// Useful for clipping? Useful for selecting multiple items?
static inline Rect GetVisibleRect (Rect a, Rect b)
{
	Rect temp;
	temp.left = (a.left < b.left) ? b.left : a.left;
	temp.right = (a.right > b.right) ? b.right : a.right;
	temp.top = (a.top < b.top) ? b.top : a.top;
	temp.bottom = (a.bottom > b.bottom) ? b.bottom : a.bottom;;
	return temp;
};
#endif
